<?php

/**
 *
 * $Id: class.AbstractTask.php,v 1.8 2005/07/14 03:27:36 dgorski Exp $
 *
 * This file is part of coWiki. coWiki is free software under the terms of
 * the GNU General Public License (GPL). Read the LICENSE file. If you did
 * not receive a copy of the license and are not able to obtain it through
 * the internet, please send a note to <license@cowiki.org> so we can mail
 * you a copy immediately.
 *
 * @package     htdocs
 * @subpackage  setup
 * @access      public
 *
 * @author      Daniel T. Gorski, <daniel.gorski@develnet.org>
 * @copyright   (C) Daniel T. Gorski, {@link http://www.develnet.org}
 * @license     http://www.gnu.org/licenses/gpl.html
 * @version     $Revision: 1.8 $
 *
 */

/**
 * Abstract task
 *
 * @package     htdocs
 * @access      public
 *
 * @author      Daniel T. Gorski, <daniel.gorski@develnet.org>
 * @since       coWiki 0.4.0
 */
abstract class AbstractTask implements TaskInterface {

    const MSG_INFO = 0;
    const MSG_WARN = 1;
    const MSG_ERR  = 2;

    protected
        $Tpl    = null,
        $State  = null,
        $aMsg   = array();

    // --------------------------------------------------------------------

    protected function __construct(InstallerTemplateProcessor $Tpl) {
        $this->Tpl   = $Tpl;
        $this->State = StateHolder::getInstance();
    }

    // --------------------------------------------------------------------

    protected function putToSession($sKey, $mValue) {
        $this->State->setKeyValueNamespace(
            get_class($this),
            $sKey,
            $mValue
        );
    }

    // --------------------------------------------------------------------

    protected function getFromSession($sKey) {
        return $this->State->getKeyValueNamespace(
                  get_class($this),
                  $sKey
               );
    }

    // --------------------------------------------------------------------

    protected function addInfo($sStr) {
        $i = sizeof($this->aMsg);
        $this->aMsg[$i]['type'] = self::MSG_INFO;
        $this->aMsg[$i]['text'] = $sStr;
        return false;
    }

    protected function addWarning($sStr) {
        $i = sizeof($this->aMsg);
        $this->aMsg[$i]['type'] = self::MSG_WARN;
        $this->aMsg[$i]['text'] = $sStr;
        return false;
    }

    protected function addWarn($sStr) {
        return $this->addWarning($sStr);
    }

    protected function addError($sStr) {
        $i = sizeof($this->aMsg);
        $this->aMsg[$i]['type'] = self::MSG_ERR;
        $this->aMsg[$i]['text'] = $sStr;
        return false;
    }

    protected function addErr($sStr) {
        return $this->addError($sStr);
    }

    // --------------------------------------------------------------------

    protected function hasError() {
        for ($i=0, $n=sizeof($this->aMsg); $i<$n; $i++) {
            if ($this->aMsg[$i]['type'] == self::MSG_ERR) {
                return true;
            }
        }

        return false;
    }

    protected function hasWarning() {
        for ($i=0, $n=sizeof($this->aMsg); $i<$n; $i++) {
            if ($this->aMsg[$i]['type'] == self::MSG_WARN) {
                return true;
            }
        }

        return false;
    }

    protected function hasMessage() {
        return $this->hasWarning() || $this->hasError();
    }

    // --------------------------------------------------------------------

    protected function getMsgQueueFormatted() {

        if (sizeof($this->aMsg) == 0) {
            return '';
        }

        $sStr =   '<table width="100%" bgcolor="#CCCCCC" cellpadding="1"';
        $sStr .=  ' cellspacing="0" border="0">';
        $sStr .=    '<tr>';
        $sStr .=      '<td width="100%">';

        $sStr .=        '<table width="100%" bgcolor="#FFFFF8"';
        $sStr .=          ' cellpadding="1" cellspacing="0" border="0">';

        for ($i=0, $n=sizeof($this->aMsg); $i<$n; $i++) {

            switch ($this->aMsg[$i]['type']) {
                case self::MSG_INFO:
                    $sImg = 'icon.info.gif';
                    break;

                case self::MSG_WARN:
                    $sImg = 'icon.warn.gif';
                    break;

                case self::MSG_ERR:
                    $sImg = 'icon.error.gif';
                    break;

                default:
                    $sImg = '0.gif';
            }

            $sStr .=  '<tr valign="top">';
            $sStr .=    '<td>';
            $sStr .=      '<img src="img/'.$sImg.'" hspace="2" vspace="1"';
            $sStr .=      ' width="16" height="16" alt="" border="0">';
            $sStr .=    '</td>';
            $sStr .=    '<td width="100%">';
            $sStr .=      $this->aMsg[$i]['text'];
            $sStr .=    '</td>';
            $sStr .=  '</tr>';
        }

        $sStr .=        '</table>';

        $sStr .=      '</td>';
        $sStr .=    '</tr>';
        $sStr .=  '</table>';

        $sStr .=  '<img src="img/0.gif" width="1" height="8"
                   alt="" border="0">';

        return $sStr;
    }

    // --------------------------------------------------------------------

    public function hasValidHtdocsDirectory() {

        // Check if we have a pointer to the 'htdocs' directory
        $sPath = $this->State->getHtdocsPath();

        // Check if files in 'htdocs' are basically accessible
        return @is_readable($sPath.'/index.php');
    }

    // --------------------------------------------------------------------

    public function hasValidIncludeDirectory() {

        // Check if we have a pointer to the 'includes' directory
        $sPath = $this->State->getIncludesPath();

        // Check if files in 'includes' are basically accessible
        return @is_readable($sPath.'/core.base.php');
    }

    // --------------------------------------------------------------------

    public function hasValidMiscDirectory() {

        // Check if we have a pointer to the 'misc' directory
        $sPath = $this->State->getMiscPath();

        // Check if files in 'misc' are basically accessible
        return @is_readable($sPath.'/database');
    }

    // --------------------------------------------------------------------

    public function hasValidClassLoader() {

        // Check if we have the class loader
        if ($this->hasValidIncludeDirectory()) {

            $sPath = $this->State->getIncludesPath();
            $sClPath = $sPath . '/class/class.ClassLoader.php';

            if (@is_readable($sClPath)) {
                $this->State->setCompleteClassLoaderPath($sClPath);
                return true;
            }
        }

        return false;
    }

    // --------------------------------------------------------------------

    // Load a valid ClassLoader of the application we are setting up. If
    // the ClassLoader is not available, jump to the beginning. FIX:
    // better solution, redirection is confusing.
    protected function loadClassLoader() {
        if (!$this->State->hasCompleteClassLoaderPath()) {
            redirectToController();
        }

        // ---

        // Load the main application ClassLoader
        include_once $this->State->getCompleteClassLoaderPath();
    }

    // --------------------------------------------------------------------

    // Check if we have a valid logged-in user (of course if this is
    // not an initial installation where a valid user is not set yet).
    // Check if user is valid.
    protected function checkValidUser() {
        if (!$this->State->isValidUser()) {
            redirectToController();
        }
    }

    // --------------------------------------------------------------------

    // Depending on if this is an inital setup or an update, we load
    // 'core.conf-dist' or 'core.conf' file.
    protected function getConfig() {
        if ($this->State->isInitialSetup()) {
            $sConf = '/core.conf-dist';
        } else {
            $sConf = '/core.conf';
        }

        return new Config($this->State->getIncludesPath() . $sConf);
    }

    // --------------------------------------------------------------------

    // Originally the schemes "mysql" and "mysqli" are not supported
    // anymore as of 0.4.0, but the maintainers decided to re-introduce
    // them again. This is a huge problem, as using "mysql" (that means
    // using MyISAM engine instead if InnoDB) will definetly lead to data
    // loss sooner or later. coWiki requires ACID transactions which
    // can not be provided my the MyISAM engine. Hands off MyISAM! (dtg)
    protected function getConnectorSchemes() {
        return array(
            'mysql+innodb'  => 'MySQL (InnoDB)',
            'mysqli+innodb' => 'MySQLi (InnoDB)',
            'mysql'         => 'MySQL (MyISAM, unsecure/destructive!)',
            'mysqli'        => 'MySQLi (MyISAM, unsecure/destructive!)',
#            'postgresql'    => 'PostgreSQL',      // Not yet implemented
#            'sqlite'        => 'SQLite',          // Not yet implemented
        );
    }

    // --------------------------------------------------------------------



} // of class

/*
    Words come easy
    Behind a screen
    When there's no interface-to-face
    To be seen

    King of insects
    You eat your own
    Atop an anthill
    You call your throne

    Kingdom of one
    So unaware
    As one-by-one your subjects
    Vanish into air

    Chatter to the wind
    Make your decree
    And save your venom
    For the ones who disagree

    Your castle walls are falling
    Your body's frail
    Your window on the world
    Is minuscule in scale

    Burrow deep now
    Escape the light
    Heaven forbid you have to face
    The ones you slight
*/

?>