<?php

/**
 *
 * $Id: class.Migrator_20050513_01.php,v 1.2 2005/07/14 03:27:55 dgorski Exp $
 *
 * This file is part of coWiki. coWiki is free software under the terms of
 * the GNU General Public License (GPL). Read the LICENSE file. If you did
 * not receive a copy of the license and are not able to obtain it through
 * the internet, please send a note to <license@cowiki.org> so we can mail
 * you a copy immediately.
 *
 * @package     htdocs
 * @subpackage  setup
 * @access      public
 *
 * @author      Daniel T. Gorski, <daniel.gorski@develnet.org>
 * @copyright   (C) Daniel T. Gorski, {@link http://www.develnet.org}
 * @license     http://www.gnu.org/licenses/gpl.html
 * @version     $Revision: 1.2 $
 *
 */

/**
 * Migrator implementation
 *
 * @package     htdocs
 * @access      public
 *
 * @author      Daniel T. Gorski, <daniel.gorski@develnet.org>
 * @since       coWiki 0.4.0
 */
class Migrator_20050513_01 implements MigratorInterface {

    private
        $State = null;

    // --------------------------------------------------------------------

    public function __construct($StateHolder) {
        $this->State = $StateHolder;
    }

    // --------------------------------------------------------------------

    /**
     *  Get a short description of what this Migrator is going to do
     */
    public function getDescription() {
        return 'Remove deprecated FULLTEXT indexes from MySQL MyISAM tables.';
    }

    // --------------------------------------------------------------------

    /**
     *  Check if this Migrator has to be performed, its task is:
     *
     *  Remove FULLTEXT indexes from legacy MySQL MyISAM tables. Hence we
     *  need to check if we are working with MySQL and if these indexes
     *  still exist.
     *
     *  This is required for the further re-definition to InnoDB.
     */
    public function isRequired() {

        // Get the resource DSN (database connection details)
        $UriInfo = new UriInfo($this->State->getDocumentStorageDSN());

        // Do we use MySQL? Check if this Migrator can/should generally
        // handle the requested connector scheme. If not, this Migrator is
        // "not required".
        switch ($UriInfo->get('scheme')) {
            case 'mysql':
            case 'mysql+innodb':
            case 'mysqli':
            case 'mysqli+innodb':
                break;

            default:
                // Not a known MySQL scheme, Migrator not required
                return false;
        }

        // Check if we have a valid storage connection with a valid
        // connector user and a valid database schema. If we do not, the
        // "basic" Migrators, those that create the connector user and
        // install the database schema were not executed yet. In this case
        // we assume that they will be executed first and report that this
        // very Migrator is required, although it was not able to check
        // its status (yet).
        if (!$this->State->hasValidDocStorageConnection()) {
            return true;
        }

        // ----------------------------------------------------------------

        // Check if the FULLTEXT indexes still exist
        $Storage = StorageFactory::getInstance()->create($UriInfo);        
        $Storage->useDatabase($UriInfo->get('basepath'));

        // ---

        // Check if table 'cowiki_comment' has FULLTEXT index
        $rResult = $Storage->query('SHOW INDEX FROM cowiki_comment');

        while ($aData = $Storage->fetchArray($rResult)) {
            if ($aData['Index_type'] == 'FULLTEXT') {
                return true;
            }
        }

        // ---

        // Check if table 'cowiki_node' has FULLTEXT index
        $rResult = $Storage->query('SHOW INDEX FROM cowiki_node');

        while ($aData = $Storage->fetchArray($rResult)) {
            if ($aData['Index_type'] == 'FULLTEXT') {
                return true;
            }
        }

        return false;
    }

    // --------------------------------------------------------------------

    /**
     *  Perform the Migrator task
     */
    public function perform() {

        // Get the connector user resource DSN (database connection details)
        $UriInfo = new UriInfo($this->State->getDocumentStorageDSN());

        // Connect to the storage/select database
        $Storage = StorageFactory::getInstance()->create($UriInfo);        
        $Storage->useDatabase($UriInfo->get('basepath'));

        // Apply changes
        try {

            $Storage->begin();

            $Storage->query('ALTER TABLE cowiki_comment DROP INDEX content');
            $Storage->query('ALTER TABLE cowiki_node DROP INDEX summary');

            $Storage->commit();

        } catch (StorageException $se) {
            // Swallow storage exceptions that might occur if INDEX is
            // already removed
        }
    }

} // of class

?>