<?php

/**
 *
 * $Id: class.AuthMySQL.php,v 1.16 2005/06/04 13:38:49 dgorski Exp $
 *
 * This file is part of coWiki. coWiki is free software under the terms of
 * the GNU General Public License (GPL). Read the LICENSE file. If you did
 * not receive a copy of the license and are not able to obtain it through
 * the internet, please send a note to <license@cowiki.org> so we can mail
 * you a copy immediately.
 *
 * @package     auth
 * @subpackage  class
 * @access      public
 *
 * @author      Daniel T. Gorski, <daniel.gorski@develnet.org>
 * @copyright   (C) Daniel T. Gorski, {@link http://www.develnet.org}
 * @license     http://www.gnu.org/licenses/gpl.html
 * @version     $Revision: 1.16 $
 *
 */

/**
 * coWiki - Auth MySQL class
 *
 * @package     auth
 * @subpackage  class
 * @access      public
 *
 * @author      Daniel T. Gorski, <daniel.gorski@develnet.org>
 * @since       coWiki 0.3.0
 *
 * @todo        [D11N]  Complete documentation
 */
class AuthMySQL extends Object implements AuthCallback {

    private
        $Storage = null;

    // --------------------------------------------------------------------

    /**
     * Init
     *
     * @access  public
     * @return  void
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     *
     * @todo    [D11N]  Check description
     */
    public function init() {

        $Registry = Registry::getInstance();
        $UriInfo = new UriInfo($Registry->get('.AUTH_RESOURCE'));

        try {
            $StorageFactory = StorageFactory::getInstance();

            // This method does not select (use) the database ...
            $this->Storage = $StorageFactory->create($UriInfo);

            // ... we need to 'use database' here
            $this->Storage->useDatabase($UriInfo->get('basepath'));

        } catch (StorageException $se) {
            RuntimeContext::getInstance()->addError(553);
        }
    }

    // --------------------------------------------------------------------

    /**
     * Validate
     *
     * @access  public
     * @param   string
     * @param   string
     * @return  mixed
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     *
     * @todo    [D11N]  Check description
     */
    public function validate($sLogin, $sPasswd) {

        // No connection, no result
        if (!is_object($this->Storage)) {
            return false;
        }

        // ----------------------------------------------------------------

        $Context = RuntimeContext::getInstance();
        $Registry = $Context->getRegistry();

        // This handler requires .AUTH_QUERY
        if (!$Registry->has('.AUTH_QUERY')) {
            $Context->addError(516, '.AUTH_QUERY');
            return false;
        }

        // This handler requires .AUTH_ENCRYPTION
        if (!$Registry->has('.AUTH_ENCRYPTION')) {
            $Context->addError(516, '.AUTH_ENCRYPTION');
            return false;
        }

        // Check if .AUTH_QUERY starts with 'SELECT '
        $sStr = strtolower(substr(trim($Registry->get('.AUTH_QUERY')),0,7));
        if ($sStr != 'select ') {
            $Context->addError(560, $Registry->get('.AUTH_QUERY'));
            return false;
        }

        // ---

        // Remove possible colons
        $sQuery = str_replace(';', '', $Registry->get('.AUTH_QUERY'));

        // Replace {%LOGIN%} placeholder
        $sQuery = str_replace('{%LOGIN%}', addslashes($sLogin), $sQuery);

        // Get user password
        try {
            $rResult = $this->Storage->query($sQuery);

        } catch (StorageException $se) {
            $Context->addError(560, $Registry->get('.AUTH_QUERY'));
            return false;
        }

        // ----------------------------------------------------------------

        $aData = $this->Storage->fetchArrayAndFreeResult($rResult);

        if ($aData) {
            $sDbPass = array_pop($aData);

            switch (strtolower($Registry->get('.AUTH_ENCRYPTION'))) {

                case 'md5':
                    return $sDbPass == md5($sPasswd);
                    break;

                case 'crypt':
                    return $sDbPass == crypt($sPasswd, $sDbPass);
                    break;

                case 'plain':
                case '':
                    return $sDbPass == $sPasswd;
                    break;
            }
        }

        return false;
    }

} // of class

?>
